#Telsa Strip Control Code v 1.2
#(C) Guy Barwood 3 Mar 2021
#May be used unrestrited for personal use.
#Use at own risk, no liability accepted
#Very little exception control in this code.
#Code may fail during untested senarios, hopefully it will just keep trying over and over till it works again.
#Update code for your Strip, this code is for 6 72pixel strips (ie: 3 144pixel strips cut in half)
#Most updates would change values under "Constants & Variables"
#Latest Changes:
#-Converted Script to use JSON functions
#-Added option for Displaying your location temperate (scaled in Celcius though, convert as desired)

import board, neopixel, time, requests, json
from datetime import datetime

#-----------Functions--------------

#---------Constants & Variables----------------
#Update these for your strips and your Gateway
NoOfPixels = 432    #Total Number of pixels in your strip
gwIP = "x.x.x.x"  #Change to YOUR local IP address of your Tesla Gateway 2
NoOfDisplays = 6    #Number of lines you have broken your strip into.  They should all have the same number of pixels.
MaxSolar = 10000    #Scale Solar.  Adjust 10000 to your max Watts of Solar you can generate: note watts, not kW
MaxBattLoad = 5000  #Battery Load Scale (Max KW displayed, 1 Powerwall 2 is max load of 5000W (5kW) in or out
MaxGridLoad = 10000 #Grid Solar.  Adjust 10000 to your max Watts of Grid Usage: note watts, not kW
MaxHomeLoad = 10000 #Max Home Load.  Adjust 10000 to your max Watts of Home Usage: note watts, not kW
PixelsPerDisplay = int(NoOfPixels/NoOfDisplays)
#Starting pixel number for each display (remember this code sees all displays as one long strip)
StartBattPixel = 0 #Display 1: Battery charge percentage (%)
StartBattLoadPixel = int(NoOfPixels)/NoOfDisplays #Display 2: Power to or from the battery
StartSolarPixel = 2*int(NoOfPixels/NoOfDisplays) #Display 3: How much solar the PVs are creating
StartLoadPixel =  3*int(NoOfPixels)/NoOfDisplays #Display 5: Load the house is currently using
StartGridPixel = 4*int(NoOfPixels)/NoOfDisplays #Display 4: Power to or from the Grid
StartVoltagePixel =  5*int(NoOfPixels)/NoOfDisplays #Display 6: Grid Voltage
StartTempPixel =  5*int(NoOfPixels)/NoOfDisplays #Display 6: Current Temperature
Display6 = "Temp" # Determines if Display 6 is Temperature or Grid Voltage: Temp = Temperatiore, Volt = Voltage
TempUpdateCount = 0 # Sets the initial status to update the Temperature on start, can comment out if displaying Voltage
TempUpdateTrigger = 60 # Update the temperature about every 60 seconds, , can comment out if displaying Voltage
CurrentTemp = 0 #Set initial Temperature to 0, , can comment out if displaying Voltage
# base URL
#To display weather you will need to create a free account on openweathermap.org and get your own API_KEY
#can comment out if displaying Voltage
BASE_URL = "https://api.openweathermap.org/data/2.5/weather?"
CITY = "YourCityName" #Find your city name on openweathermap.org
API_KEY = "change_to_your_api_key"
# Create the Weather URL
URL = BASE_URL + "q=" + CITY + "&units=metric&appid=" + API_KEY #change units to imperial if you want F, but change display scale in code.

#Powerwall 2 Credentials
Password = "password" #password you log into your powerwall with
Email = "no@email.com" #email doesn't seem to matter at the moment, change to your log in email if logins fail in the future

#-----------Code-------------------      
pixels = neopixel.NeoPixel(board.D18, NoOfPixels, auto_write=False) #Setup Light String
requests.packages.urllib3.disable_warnings() #Disable SSL Warnings for Gatway self-signed certificate
print("Running: Press CONTROL+C to terminate.")

#Reset Pixels off to start with
for x in range(1,int(NoOfPixels)):
    pixels[x] = (0, 0, 0) #All other pixels off
pixels.show() #Update the light strip with new values
time.sleep(1)

#Try to login to the Powerwall and create session object
#dissabled for now and not needed
headers = {
    'Content-Type': 'application/json',
}
data = '{"username":"customer","password":"' + Password + '", "email":"' + Email + '","force_sm_off":false}'
s = requests.Session()
s.post("https://" + gwIP + "/api/login/Basic", headers=headers, data=data, verify=False)



while True :
    try:       
        #Get and display Temperature by JSON
        response=s.get("https://" + gwIP + "/api/system_status/soe", verify=False)
        data = response.json()
        BatteryPercentFull = data["percentage"]
        if BatteryPercentFull == 100: #special case for exactly 100%
            for x in range(0, PixelsPerDisplay):
              pixels[x] = (0, 1, 0) #Display all as green
            #pixels.show()
        else:
            NoPixels = round(BatteryPercentFull * PixelsPerDisplay / 100,0)
            for x in range(int(NoPixels)):
              pixels[x] = (1, 0, 0) #display charge as red
            for x in range((int(NoPixels)),PixelsPerDisplay):
              pixels[x] = (0, 0, 0) #display above charge as teal
        
        #Get other Data
        response = s.get("https://" + gwIP + "/api/meters/aggregates", verify=False)
        data = response.json()
        #Seperate out JSON Libraries
        Site = data['site']
        Battery = data['battery']
        Load = data['load']
        Solar = data['solar']
        
        #Display 2: Current Charge or Load on the battery
        IBattLoad = Battery["instant_power"]      
        if IBattLoad == 0: #Special case for exactly battery is full and not drawing load or charging
            for x in range(int(StartBattLoadPixel), int(StartBattLoadPixel+PixelsPerDisplay)):
              pixels[x] = (0, 0, 0) #Display all as off
        else:
            NoPixels = abs(round(((int(IBattLoad)/int(MaxBattLoad)) * PixelsPerDisplay),0)) #calculate number of pixels to display
            if int(IBattLoad) < 0: #charging
                for x in range(int(StartBattLoadPixel), int(StartBattLoadPixel+NoPixels)):
                  pixels[x] = (0, 1, 0) #display charge as green            
            else: #IBattLoad > 0 so is charging
                for x in range(int(StartBattLoadPixel), int(StartBattLoadPixel+NoPixels)):
                  pixels[x] = (1, 0, 0) #display charge as red
            for x in range(int(StartBattLoadPixel+NoPixels), int(StartBattLoadPixel++PixelsPerDisplay)):
                pixels[x] = (0, 0, 0) #display remaining pixels as teal   

        #Display 3 : Get Current Solar Generation
        ISolar = Solar["instant_power"]
        NoPixels = round(((int(ISolar)/int(MaxSolar)) * PixelsPerDisplay),0)
        for x in range(int(StartSolarPixel), int(StartSolarPixel+NoPixels)):
            pixels[x] = (1, 1, 0)
        for x in range((int(StartSolarPixel+NoPixels)),int(StartSolarPixel+PixelsPerDisplay)):
            pixels[x] = (0, 0, 0)

        #Display 4
        #Get Current Home Load & Display
        IPower = Load["instant_power"]
        NoPixels = round(((int(IPower)/int(MaxHomeLoad)) * PixelsPerDisplay),0)
        for x in range(int(StartLoadPixel), int(StartLoadPixel+NoPixels)):
            pixels[x] = (1, 1, 0) #Display home load as Yellow
        for x in range((int(StartLoadPixel+NoPixels)),int(StartLoadPixel+PixelsPerDisplay)):
            pixels[x] = (0, 0, 0) #Display above home load as off
    
        #Display 5 : Get Current Grid Load (To/From Grid)
        ISite = Site["instant_power"]
        NoPixels = round(((abs(int(ISite)/int(MaxGridLoad)) * PixelsPerDisplay)),0)
        if int(ISite) < 0:
            for x in range(int(StartGridPixel), int(StartGridPixel+NoPixels)):
                pixels[x] = (0, 1, 0) #Green = Feed Into Grid
        else:
            for x in range(int(StartGridPixel), int(StartGridPixel+NoPixels)):
                pixels[x] = (1, 0, 0) #Green = Feed From Grid
        for x in range((int(StartGridPixel+NoPixels)),int(StartGridPixel+PixelsPerDisplay)):
            pixels[x] = (0, 0, 0) #All other pixels off
       
        #Display 6
        if Display6 == "Volt":  #Display 6: Current Voltage (Scaled 240-260v)
            IVoltage = Battery["instant_average_voltage"]
            NoPixels = round((((int(IVoltage)-235)/20) * PixelsPerDisplay),0) #Scale 235-260v
            if NoPixels < 0: # lower than 240v set all off
                for x in range(int(StartVoltagePixel), int(StartVoltagePixel+PixelsPerDisplay)):
                    pixels[x] = (0, 0, 0) #display off
            else: #Voltage > 240v
                if NoPixels > int(PixelsPerDisplay): # greater than 260v set all RED
                    for x in range(int(StartVoltagePixel), int(StartVoltagePixel+PixelsPerDisplay)):
                      pixels[x] = (1, 0, 0) #display charge as red
                else: #voltage in range 240-260v
                    for x in range(int(StartVoltagePixel), int(StartVoltagePixel+NoPixels)):
                        pixels[x] = (0, 1, 0) #Display home load as green
                    for x in range(int(StartVoltagePixel+NoPixels), int(StartVoltagePixel+PixelsPerDisplay)):
                        pixels[x] = (0, 0, 0) #display remaining pixels as teal  
        elif Display6 == "Temp":    #Display 6: Display Current Temperature sourced from openweathermap.org
            if TempUpdateCount > 0 and TempUpdateCount < TempUpdateTrigger: #do not update temperature yet
                TempUpdateCount += 1 #increment count towards trigger
            else: #time to update the temperaturee
                # HTTP request
                response = requests.get(URL)
                # checking the status code of the request
                if response.status_code == 200: #success
                   # getting data in the json format
                   data = response.json()
                   # getting the main dict block
                   main = data['main']
                   # getting temperature
                   CurrentTemp = main['temp']
                   TempUpdateCount = 1 #Reset count to 1 (not 0, 0 is only for the first update)
                else: #if failed to get weather set temp to 0
                    CurrentTemp = 0
            #Display Temp from scale 0-50            
            NoPixels = round((((int(CurrentTemp))/50) * PixelsPerDisplay),0) #Scale 0-50c
            if NoPixels == 0: # = 0c or lower set all off
                for x in range(int(StartTempPixel), int(StartTempPixel+PixelsPerDisplay)):
                    pixels[x] = (0, 0, 0) #display off
            else: #Temp > 0c
                if NoPixels > int(PixelsPerDisplay): # greater than 50c set all RED
                    for x in range(int(StartTempPixel), int(StartTempPixel+PixelsPerDisplay)):
                      pixels[x] = (5, 0, 0) #display charge as red
                else: #voltage in range 1-50c
                    #breaking display into 10c segments
                    for x in range(int(StartTempPixel), int(StartTempPixel+NoPixels)):
                        if x <= StartTempPixel+14: #0-10c = 14.4pixels
                            pixels[x] = (0, 0, 3) #Display home load as brighter blue
                        elif x > StartTempPixel+14 and x <= StartTempPixel+29: #11-20c = 28.8pixels
                            pixels[x] = (0, 0, 1) #Display home load as lower blue
                        elif x > StartTempPixel+29 and x <= StartTempPixel+43: #21-30c = 43.2pixels
                            pixels[x] = (0, 1, 0) #Display home load as green
                        elif x > StartTempPixel+43 and x <= StartTempPixel+58: #31-40c = 57.6pixels
                            pixels[x] = (1, 0, 0) #Display home load as lower red                            
                        elif x > StartTempPixel+58: #41-50c
                            pixels[x] = (3, 0, 0) #Display home load as brighter red
                    for x in range(int(StartTempPixel+NoPixels), int(StartTempPixel+PixelsPerDisplay)):
                        pixels[x] = (0, 0, 0) #display remaining pixels as off
                    
        pixels.show() #Update the light strip with new values
        time.sleep(1) #Refresh interval in seconds(+ time of code execution)

    except Exception as e:  #if an exception occurs, wait 1 second and try again.  Still allows CONTROL-C to stop execution.
        print("Exception raised :" + str(e)) #Write exception to console (for intertactive session troubleshooting)
        time.sleep(1) #sleep and then try to get the data again.
        #Turn all the pixels off
        for x in range(1,int(NoOfPixels)):
            pixels[x] = (0, 0, 0) #All other pixels off
        pixels.show() #Update the light strip with new values
        #Below code for logging into Powerwall if required.
        #End login session and try to establish a new session
        #s.close()
        #s = requests.Session()
        #s.post("https://" + gwIP + "/api/login/Basic", headers=headers, data=data, verify=False)

